from abc import ABC, abstractmethod


class SpeechToText(ABC):

    @abstractmethod
    def recognize(self, audio, language):
        pass

    @abstractmethod
    def recognize_from_url(self, audio_url, language):
        pass



from .assistant_exception import AssistantException
from .watson_assistant import WatsonAssistant


class AssistantFactory:

    @staticmethod
    def create(service_type, credentials):

        if service_type is None:
            raise AssistantException(code=400, message='Missing service_type parameter')

        if service_type.lower() == 'watson':
            return WatsonAssistant(credentials)
        else:
            raise AssistantException(code=400, message=f'Type {service_type} is not supported')



from .speech_to_text_factory import SpeechToTextFactory
from .speech_to_text_web_socket_factory import SpeechToTextWebSocketFactory
from .speech_to_text_exception import SpeechToTextException

__all__ = [
    'SpeechToTextFactory',
    'SpeechToTextWebSocketFactory',
    'SpeechToTextException'
]



import requests
from watson_developer_cloud import SpeechToTextV1, watson_service
from .speech_to_text import SpeechToText
from .speech_to_text_exception import SpeechToTextException


class WatsonSpeechToText(SpeechToText):

    def __init__(self, credentials):
        username = credentials.get('username')
        password = credentials.get('password')
        errors = []

        if not username:
            errors.append("username")
        if not password:
            errors.append("password")

        if len(errors) > 0:
            raise SpeechToTextException(code=400, message='Missing parameters: {0}'.format(', '.join(errors)))

        self.speechToText = SpeechToTextV1(
            username=username,
            password=password
        )

    def recognize(self, audio, language):
        errors = []

        if not language:
            errors.append("language")
        if not audio:
            errors.append("audio")

        if len(errors) > 0:
            raise SpeechToTextException(code=400, message='Missing parameters: {0}'.format(', '.join(errors)))

        if language == 'en':
            model = 'en-US_BroadbandModel'
        elif language == 'pt-br':
            model = 'pt-BR_NarrowbandModel'
        else:
            raise SpeechToTextException(code=400, message='Invalid language value')

        return self.speechToText.recognize(
            audio=audio,
            model=model
        )

    def recognize_from_url(self, audio_url, language):
        errors = []

        if not language:
            errors.append("language")
        if not audio_url:
            errors.append("audio_url")

        if len(errors) > 0:
            raise SpeechToTextException(code=400, message='Missing parameters: {0}'.format(', '.join(errors)))

        if language == 'en':
            model = 'en-US_BroadbandModel'
        elif language == 'pt-br':
            model = 'pt-BR_NarrowbandModel'
        else:
            raise SpeechToTextException(code=400, message='Invalid language value')

        try:
            r = requests.get(audio_url)

            if r.status_code == 200:
                return self.speechToText.recognize(
                    audio=r.content,
                    model=model
                )
            else:
                raise SpeechToTextException(code=r.status_code, message=f'Problem downloading file {audio_url}')
        except watson_service.WatsonApiException as ex:
            raise SpeechToTextException(code=ex.code, message=ex.message)
        except Exception as ex:
            raise SpeechToTextException(code=500, message=ex.args[0])



import base64
import json
#from googleapiclient.discovery import build
from .speech_to_text import SpeechToText
from .speech_to_text_exception import SpeechToTextException


class GoogleSpeechToText(SpeechToText):

    def __init__(self, credentials):
        api_key = credentials.get('api_key')

        if not api_key:
            raise SpeechToTextException(code=400, message='Missing parameters: api_key')

        #self.speechToText = build('speech', 'v1', developerKey=api_key)

    def recognize(self, audio, language):

        errors = []
        if not language:
            errors.append("language")
        if not audio:
            errors.append("audio")

        if len(errors) > 0:
            raise SpeechToTextException(code=400, message='Missing parameters: {0}'.format(', '.join(errors)))

        if language == 'en':
            language_code = 'en-US'
        elif language == 'pt-br':
            language_code = 'pt-BR'
        else:
            raise SpeechToTextException(code=400, message='Invalid language value')

        body = {
            'audio': {
                'content': base64.b64encode(audio.read()).decode('utf-8')
            },
            'config': {
                'languageCode': language_code
            }
        }

        # try:
        #     return self.speechToText.speech().recognize(
        #         body=body
        #     ).execute()
        # except Exception as ex:
        #     error = json.loads(ex.content).get('error')
        #     if error:
        #         raise SpeechToTextException(code=error['code'], message=error['message'])
        #     else:
        #         raise SpeechToTextException(code=500, message='Unknown error')

    def recognize_from_url(self, audio_url, language):
        pass



from abc import ABC, abstractmethod


class Assistant(ABC):

    @abstractmethod
    def message(self, parameters):
        pass

    @abstractmethod
    def information(self, parameters):
        pass



import websocket
import json
from threading import Thread
from watson_developer_cloud import AuthorizationV1
from .speech_to_text_web_socket import SpeechToTextWebSocket


class WatsonSpeechToTextWebSocket(SpeechToTextWebSocket):

    __URI__ = 'wss://stream.watsonplatform.net/speech-to-text/api/v1/recognize'

    def __init__(self, consumer, credentials, language, frame_rate):
        Thread.__init__(self)
        SpeechToTextWebSocket.__init__(self, consumer, credentials, language, frame_rate)

    def run(self):
        print("Starting thread")

        auth = AuthorizationV1(username='b0398c98-75e8-4628-ac72-58722de24946',
                               password='5UAUkSXZhNuv')

        token = auth.get_token('https://stream.watsonplatform.net/speech-to-text/api')

        url = '{0}?watson-token={1}&model={2}'.format(self.__URI__, token, 'pt-BR_NarrowbandModel')

        self.ws = websocket.WebSocketApp(url,
                                         on_open=self.on_open,
                                         on_message=self.on_message,
                                         on_error=self.on_error,
                                         on_close=self.on_close)
        self.ws.keep_running = True
        self.ws.daemon = True
        self.ws.run_forever()

    def start_client(self):
        message = {
            'action': 'start',
            'content-type': 'audio/wav'
        };
        self.ws.send(json.dumps(message), websocket.ABNF.OPCODE_TEXT)
        self.ws.send(SpeechToTextWebSocket.get_wave_header(self.nchannels, self.sample_width, self.frame_rate),
                     opcode=websocket.ABNF.OPCODE_BINARY)
        if self.bytes:
            self.send_data(self.bytes)

    def send_data(self, data):
        if not self.is_open:
            self.bytes += data
        elif self.bytes:
            self.ws.send(self.bytes, opcode=websocket.ABNF.OPCODE_BINARY)
            self.bytes = b''
        else:
            self.ws.send(data, opcode=websocket.ABNF.OPCODE_BINARY)

    def close_client(self):
        message = {
            'action': 'stop'
        }
        self.ws.send(json.dumps(message), websocket.ABNF.OPCODE_TEXT)

    def stop(self):
        print('stopping')
        self.ws.keep_running = False






class SpeechToTextException(Exception):

    def __init__(self, code, message):
        self.code = code
        self.message = message

    def __str__(self):
        msg = 'Error: ' + str(self.message) + ', Code: ' + str(self.code)
        return msg



import json
import wave
from io import BytesIO

from .google_speech_to_text import GoogleSpeechToText
from .speech_to_text_web_socket import SpeechToTextWebSocket
from .speech_to_text_exception import SpeechToTextException


class GoogleSpeechToTextWebSocket(SpeechToTextWebSocket):

    def __init__(self, consumer, credentials, language, frame_rate):
        SpeechToTextWebSocket.__init__(self, consumer, credentials, language, frame_rate)
        self.mem_audio = None
        self.wave = None

    def run(self):
        print("Starting thread")

        self.mem_audio = BytesIO()
        self.wave = wave.open(self.mem_audio, 'wb')
        self.wave.setnchannels(1)
        self.wave.setsampwidth(2)
        self.wave.setframerate(self.frame_rate)

    def start_client(self):
        pass

    def send_data(self, data):
        self.wave.writeframes(data)

    def close_client(self):
        self.wave.close()
        self.mem_audio.seek(0)

        try:
            print('sending audio to speech to text service')
            speech_to_text = GoogleSpeechToText(self.credentials)
            response = speech_to_text.recognize(
                audio=self.mem_audio,
                language=self.language
            )

            if 'results' in response and len(response['results']) > 0 and \
                    'alternatives' in response['results'][0] and len(response['results'][0]['alternatives']) > 0 and \
                    'transcript' in response['results'][0]['alternatives'][0]:
                self.consumer.send(json.dumps(response['results'][0]['alternatives'][0]))
            else:
                self.consumer.send(json.dumps({'error': 'Não foi possível reconhecer o audio enviado. Tente novamente.'}))

        except SpeechToTextException as ex:
            response = {
                'error': ex.message
            }
        except Exception as ex:
            response = {
                'error': ex.args[0]
            }
        self.consumer.send(json.dumps(response))



from .speech_to_text_exception import SpeechToTextException
from .google_speech_to_text_web_socket import GoogleSpeechToTextWebSocket
from .watson_speech_to_text_web_socket import WatsonSpeechToTextWebSocket


class SpeechToTextWebSocketFactory:

    @staticmethod
    def create(service_type, credentials):

        if service_type is None:
            raise SpeechToTextException(code=400, message='Missing service_type parameter')

        if service_type.lower() == 'google':
            return GoogleSpeechToTextWebSocket(credentials)
        elif service_type.lower() == 'watson':
            return WatsonSpeechToTextWebSocket(credentials)
        else:
            raise SpeechToTextException(code=400, message=f'Type {service_type} is not supported')



import struct
import json
from io import BytesIO
from abc import ABC, abstractmethod
from threading import Thread


class SpeechToTextWebSocket(ABC, Thread):
    @abstractmethod
    def __init__(self, consumer, credentials, language, frame_rate):
        Thread.__init__(self)
        self.ws = None
        self.is_open = False
        self.consumer = consumer
        self.credentials = credentials
        self.language = language
        self.frame_rate = frame_rate
        self.nchannels = 1
        self.sample_width = 2
        self.bytes = b''

    @abstractmethod
    def run(self):
        pass

    @abstractmethod
    def start_client(self):
        pass

    @abstractmethod
    def send_data(self, data):
        pass

    @abstractmethod
    def close_client(self):
        pass

    def on_message(self, ws, message):

        message_json = json.loads(message)

        if 'results' in message_json:
            if len(message_json['results']) > 0 and \
                'alternatives' in message_json['results'][0] and len(message_json['results'][0]['alternatives']) > 0 \
                    and 'transcript' in message_json['results'][0]['alternatives'][0]:
                self.consumer.send((json.dumps(message_json['results'][0]['alternatives'][0])))
            else:
                self.consumer.send(json.dumps({'error': 'Não foi possível reconhecer o audio enviado. Tente novamente.'}))
        else:
            self.consumer.send(message)

    def on_error(self, ws, error):
        print(error)
        self.consumer.send(json.dumps({'error': error}))

    def on_close(self, ws):
        print("### closed ###")

    def on_open(self, ws):
        print('on_open')
        self.start_client()
        self.is_open = True

    @staticmethod
    def get_wave_header(nchannels, sample_width, frame_rate):

        output = BytesIO()
        output.write(struct.pack('>L', 0x52494646))
        output.write(struct.pack('<L', 0))
        output.write(struct.pack('>L', 0x57415645))
        output.write(struct.pack('>L', 0x666d7420))
        output.write(struct.pack('<L', 16))
        output.write(struct.pack('<H', 0x0001))
        output.write(struct.pack('<H', nchannels))
        output.write(struct.pack('<L', frame_rate))
        output.write(struct.pack('<L', frame_rate * nchannels * sample_width))
        output.write(struct.pack('<H', nchannels * sample_width))
        output.write(struct.pack('<H', sample_width * 8))
        output.write(struct.pack('>L', 0x64617461))
        output.write(struct.pack('<L', 0))

        data = output.getvalue()

        output.close()

        return data




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cognitive-services",
    version="0.0.1",
    author="Franklin Lindemberg Guimaraes",
    author_email="franklin.lindemberg@seedin.ai",
    description="A package that wraps many cognitive services packages",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/franklin.lindemberg/cognitive-services",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'watson-developer-cloud>=1.3.2,<2',
        'google-api-python-client>=1.7.0,<2',
        'websocket-client>=0.48.0,<1',
        'requests>=2.21.0,<3'
    ],

)


import json
from watson_developer_cloud import AssistantV1, watson_service
from .assistant import Assistant
from .assistant_exception import AssistantException


class WatsonAssistant(Assistant):

    def __init__(self, credentials):
        username = credentials.get('username')
        password = credentials.get('password')

        errors = []
        if not username:
            errors.append("username")
        if not password:
            errors.append("password")

        if len(errors) > 0:
            raise AssistantException(code=400, message='Missing parameters: {0}'.format(', '.join(errors)))

        self.conversation = AssistantV1(
            username=username,
            password=password,
            version='2018-07-10'
        )

    def message(self, parameters):

        workspace_id = parameters.get('workspace_id')
        body = parameters.get('body')

        errors = []
        if not workspace_id:
            errors.append("workspace_id")
        if not body:
            errors.append("body")

        if len(errors) > 0:
            raise AssistantException(code=400, message='Missing parameters: {0}'.format(', '.join(errors)))

        try:
            body = json.loads(body)
            input_text = body.get('input')
            context = body.get('context')

            response = self.conversation.message(
                workspace_id=workspace_id,
                input=input_text,
                context=context,
                alternate_intents=True
            )
            return response
        except watson_service.WatsonApiException as ex:
            raise AssistantException(code=ex.code, message=ex.message)
        except Exception as ex:
            raise AssistantException(code=500, message=ex.args[0])

    def information(self, parameters):

        workspace_id = parameters.get('workspace_id', None)

        if not workspace_id:
            raise AssistantException(code=400, message='Missing parameter: workspace_id')

        try:
            response = self.conversation.get_workspace(
                workspace_id=workspace_id,
                export=False
            )
            return response
        except watson_service.WatsonApiException as ex:
            raise AssistantException(code=ex.code, message=ex.message)
        except Exception as ex:
            raise AssistantException(code=500, message=ex.args[0])
